"""This module is for the translation mapping data of the
following document:

Document file: datetime.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DateTime class": "# DateTime クラス",
    ##################################################
    "This page explains the `DateTime` class.": "このページでは`DateTime`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `DateTime` class is a class to handle each date- and time-related interface.": "`DateTime`クラスは日付と時間かんけぽの各インターフェイスを扱うためのクラスです。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The constructor requires the `year`, `month`, and `day` arguments, as follows:": "コンストラクタの引数では`year`、`month`、`day`の各引数の指定を必要とします。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndt: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)\n```": "```py\n# runnable\nimport apysc as ap\n\ndt: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)\n```",  # noqa
    ##################################################
    "Also, it has the optional `hour`, `minute`, `second`, and `millisecond` arguments:": "また、省略可能な引数として`hour`、`minute`、`second`の引数を指定することもできます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndt: ap.DateTime = ap.DateTime(\n    year=2022, month=12, day=5, hour=10, minute=30, second=50, millisecond=500\n)\n```": "```py\n# runnable\nimport apysc as ap\n\ndt: ap.DateTime = ap.DateTime(\n    year=2022, month=12, day=5, hour=10, minute=30, second=50, millisecond=500\n)\n```",  # noqa
    ##################################################
    "Each value has a getter and setter interface.": "各値はgetterとsetterのインターフェイスを持っています。",  # noqa
    ##################################################
    "For instance, the `month` value can get or set via the properties:": "例えば`month`属性であれば以下のように属性経由で値の取得や更新を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndt: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)\nassert dt.month == 12\ndt.month = ap.Int(10)\nassert dt.month == 10\n```": "```py\n# runnable\nimport apysc as ap\n\ndt: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)\nassert dt.month == 12\ndt.month = ap.Int(10)\nassert dt.month == 10\n```",  # noqa
    ##################################################
    "Notes: the `weekday_py` and `weekday_js` properties only have a getter interface.": "特記事項: `weekday_py`と`weekday_js`属性はgetterのインターフェイスのみ存在します。",  # noqa
    ##################################################
    "For more information about the other properties, please see the followings:": "他の属性の詳細に関しては以下のを参照してください:",  # noqa
    ##################################################
    "- [DateTime class year property](datetime_year.md)": "- [DateTime クラスの year 属性](jp_datetime_year.md)",  # noqa
    ##################################################
    "- [DateTime class month property](datetime_month.md)": "- [DateTime クラスの month 属性](jp_datetime_month.md)",  # noqa
    ##################################################
    "- [DateTime class day property](datetime_day.md)": "- [DateTime クラスの day 属性](jp_datetime_day.md)",  # noqa
    ##################################################
    "- [DateTime class hour property](datetime_hour.md)": "- [DateTime クラスの hour 属性](jp_datetime_hour.md)",  # noqa
    ##################################################
    "- [DateTime class minute property](datetime_minute.md)": "- [DateTime クラスの minute 属性](jp_datetime_minute.md)",  # noqa
    ##################################################
    "- [DateTime class second property](datetime_second.md)": "- [DateTime クラスの second 属性](jp_datetime_second.md)",  # noqa
    ##################################################
    "- [DateTime class millisecond property](datetime_millisecond.md)": "- [DateTime クラスの millisecond 属性](jp_datetime_millisecond.md)",  # noqa
    ##################################################
    "- [DateTime class weekday_js and weekday_py properties](datetime_weekday_js_and_weekday_py.md)": "- [DateTime クラスの weekday_js と weekday_py 属性](jp_datetime_weekday_js_and_weekday_py.md)",  # noqa
    ##################################################
    "Also, the `DateTime` class has each method interface, such as the `now` class method.": "また、`DateTime`クラスは`now`のクラスメソッドのような各メソッドのインターフェイスを持っています。",  # noqa
    ##################################################
    "For more information about the methods interfaces, please see the following:": "メソッドの各インターフェイスの詳細は以下を参照してください。",  # noqa
    ##################################################
    "- [DateTime class now interface](datetime_now.md)": "- [DateTime クラスの now インターフェイス](jp_datetime_now.md)",  # noqa
    ##################################################
    "## DateTime class constructor API": "## DateTime クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The class for datetime-related interfaces.<hr>": "日時に絡んだインターフェイスのためのクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `year`: Union[int, Int]": "- `year`: Union[int, Int]",
    ##################################################
    "  - Four-digit year.": "  - 4桁の数字の年。",
    ##################################################
    "- `month`: Union[int, Int]": "- `month`: Union[int, Int]",
    ##################################################
    "  - Two-digit month (1 to 12).": "  - 2桁の月（1～12）。",
    ##################################################
    "- `day`: Union[int, Int]": "- `day`: Union[int, Int]",
    ##################################################
    "  - Two-digit day (1 to 31).": "  - 2桁の日（1～31）。",
    ##################################################
    "- `hour`: Optional[Union[int, Int]], optional": "- `hour`: Optional[Union[int, Int]], optional",  # noqa
    ##################################################
    "  - Two-digit hour (0 to 23).": "  - 2桁の時（0～23）。",
    ##################################################
    "- `minute`: Optional[Union[int, Int]], optional": "- `minute`: Optional[Union[int, Int]], optional",  # noqa
    ##################################################
    "  - Two-digit minute (0 to 59).": "  - 2桁の分（0～59）。",
    ##################################################
    "- `second`: Optional[Union[int, Int]], optional": "- `second`: Optional[Union[int, Int]], optional",  # noqa
    ##################################################
    "  - Two-digit second (0 to 59).": "  - 2桁の秒（0～59）。",
    ##################################################
    "- `millisecond`: Optional[Union[int, Int]], optional": "- `millisecond`: Optional[Union[int, Int]], optional",  # noqa
    ##################################################
    "  - Millisecond (0 to 999).": "  - ミリ秒（0～999）",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "- `skip_init_substitution_expression_appending`: bool, default False": "- `skip_init_substitution_expression_appending`: bool, default False",  # noqa
    ##################################################
    "  - A boolean indicates whether to skip an initial substitution expression or not. The `DateTime` class uses this option internally.": "  - 初期値の代入表現の追加をスキップするかどうかの真偽値。`DateTime`クラスでは内部でのみこのオプションを使用します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> datetime_: ap.DateTime = ap.DateTime(\n...     year=2022,\n...     month=12,\n...     day=5,\n...     hour=10,\n...     minute=30,\n...     second=50,\n...     millisecond=500,\n... )\n>>> datetime_.year\nInt(2022)\n\n>>> datetime_.month\nInt(12)\n\n>>> datetime_.day\nInt(5)\n\n>>> datetime_.hour\nInt(10)\n\n>>> datetime_.minute\nInt(30)\n\n>>> datetime_.millisecond\nInt(500)\n\n>>> datetime_.weekday_py\nInt(0)\n\n>>> datetime_.weekday_js\nInt(1)\n```": "```py\n>>> import apysc as ap\n>>> datetime_: ap.DateTime = ap.DateTime(\n...     year=2022,\n...     month=12,\n...     day=5,\n...     hour=10,\n...     minute=30,\n...     second=50,\n...     millisecond=500,\n... )\n>>> datetime_.year\nInt(2022)\n\n>>> datetime_.month\nInt(12)\n\n>>> datetime_.day\nInt(5)\n\n>>> datetime_.hour\nInt(10)\n\n>>> datetime_.minute\nInt(30)\n\n>>> datetime_.millisecond\nInt(500)\n\n>>> datetime_.weekday_py\nInt(0)\n\n>>> datetime_.weekday_js\nInt(1)\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [DateTime class year property](https://simon-ritchie.github.io/apysc/en/datetime_year.html)": "- [DateTime クラスの year 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_year.html)",  # noqa
    ##################################################
    "- [DateTime class month property](https://simon-ritchie.github.io/apysc/en/datetime_month.html)": "- [DateTime クラスの month 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_month.html)",  # noqa
    ##################################################
    "- [DateTime class day property](https://simon-ritchie.github.io/apysc/en/datetime_day.html)": "- [DateTime クラスの day 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_day.html)",  # noqa
    ##################################################
    "- [DateTime class hour property](https://simon-ritchie.github.io/apysc/en/datetime_hour.html)": "- [DateTime クラスの hour 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_hour.html)",  # noqa
    ##################################################
    "- [DateTime class minute property](https://simon-ritchie.github.io/apysc/en/datetime_minute.html)": "- [DateTime クラスの minute 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_minute.html)",  # noqa
    ##################################################
    "- [DateTime class second property](https://simon-ritchie.github.io/apysc/en/datetime_second.html)": "- [DateTime クラスの second 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_second.html)",  # noqa
    ##################################################
    "- [DateTime class millisecond property](https://simon-ritchie.github.io/apysc/en/datetime_millisecond.html)": "- [DateTime クラスの millisecond 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_millisecond.html)",  # noqa
    ##################################################
    "- [DateTime class weekday_js and weekday_py properties](https://simon-ritchie.github.io/apysc/en/datetime_weekday_js_and_weekday_py.html)": "- [DateTime クラスの weekday_js と weekday_py 属性](https://simon-ritchie.github.io/apysc/jp/jp_datetime_weekday_js_and_weekday_py.html)",  # noqa
    ##################################################
    "- [DateTime class now interface](https://simon-ritchie.github.io/apysc/en/datetime_now.html)": "- [DateTime クラスの now インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_datetime_now.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: int_and_number_comparison_operations.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Int and Number basic comparison operations": "# Int と Number クラスの基本的な比較制御",
    ##################################################
    "This page explains basic comparison operations of the `Int` and `Number` classes, like the `>=`\\, `<`\\.": "このページでは`Int`や`Number`クラスの`>=`や`<`などの基本的な比較制御について説明します。",  # noqa
    ##################################################
    "## Common behaviors": "## 共通の挙動",
    ##################################################
    "Each comparison operation returns a `Boolean` value, not a Python built-in `bool` value:": "各比較制御はPythonのビルトインの`bool`の値ではなく`Boolean`の値を返却します。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == 10\nassert isinstance(result, ap.Boolean)\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == 10\nassert isinstance(result, ap.Boolean)\n```",  # noqa
    ##################################################
    "You can compare the `Int` or `Number` values with the Python built-in values, like the `int` or `float`:": "`Int`や`Number`の値をPythonビルトインの`int`や`float`などの値と比較することもできます:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(20)\nresult: ap.Boolean = int_1 == 20\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(20)\nresult: ap.Boolean = int_1 == 20\nassert result\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nresult: ap.Boolean = number_1 == 10.5\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nresult: ap.Boolean = number_1 == 10.5\nassert result\n```",  # noqa
    ##################################################
    "Also, the comparison between the `Int` and `Int`\\, `Number` and `Number`\\, `Int` and `Number` are supported:": "同様に`Int`と`Int`間、`Number`と`Number`間、`Int`と`Number`間の比較などもサポートしています:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == int_2\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == int_2\nassert result\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nnumber_2: ap.Number = ap.Number(10.5)\nresult: ap.Boolean = number_1 == number_2\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nnumber_2: ap.Number = ap.Number(10.5)\nresult: ap.Boolean = number_1 == number_2\nassert result\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = ap.Number(10)\nresult: ap.Boolean = int_1 == number_1\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = ap.Number(10)\nresult: ap.Boolean = int_1 == number_1\nassert result\n```",  # noqa
    ##################################################
    "## Equal comparison operator": "## 等値条件の比較のオペレーター",
    ##################################################
    "You can use the `==` operator for the equal comparison:": "`==`のオペレーターを使って等値条件の比較を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == 10\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == 10\nassert result\n```",  # noqa
    ##################################################
    "## Not equal comparison operator": "## 非等値条件の比較のオペレーター",
    ##################################################
    "You can use the `!=` operator for the not equal comparison:": "`!=`のオペレーターを使って非等値条件の比較を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 != 15\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 != 15\nassert result\n```",  # noqa
    ##################################################
    "## Less than comparison operator": "## 未満条件の比較のオペレーター",
    ##################################################
    "You can use the `<` operator for the less than comparison:": "`<`のオペレーターを使って未満条件の比較を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 < 11\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 < 11\nassert result\n```",  # noqa
    ##################################################
    "## Less than or equal comparison operator": "## 以下条件の比較のオペレーター",
    ##################################################
    "You can use the `<=` operator for the less than or equal comparison:": "`<=`のオペレーターを使って以下条件の比較を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 <= 10\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 <= 10\nassert result\n```",  # noqa
    ##################################################
    "## Greater than comparison operator": "## 超過条件の比較のオペレーター",
    ##################################################
    "You can use the `>` operator for the greater than comparison:": "`>`のオペレーターを使って超過条件の比較を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 > 9\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 > 9\nassert result\n```",  # noqa
    ##################################################
    "## Greater than or equal comparison operator": "## 以上条件の比較のオペレーター",
    ##################################################
    "You can use the `>=` operator for the greater than or equal comparison:": "`>=`のオペレーターを使って以上条件の比較を行うことができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 >= 10\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 >= 10\nassert result\n```",  # noqa
}



"""Mix-in class implementation for the first control x path data.
"""

from typing import Dict

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class PathControlX1MixIn(
    VariableNameSuffixAttrMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
    VariableNameSuffixMixIn,
):

    _control_x1: Int

    @final
    def _initialize_control_x1_if_not_initialized(self) -> None:
        """
        Initialize the _control_x1 attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, "_control_x1"):
            return
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="control_x1")
        self._control_x1 = Int(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_control_x1_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_control_x1_linking_setting(self) -> None:
        """
        Append a control_x1 attribute to the linking setting.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._control_x1, attr_name="control_x1"
        )
        self._append_attr_to_linking_stack(
            attr=self._control_x1, attr_name="control_x1"
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def control_x1(self) -> Int:
        """
        Get a first x-coordinate of the control point.

        Returns
        -------
        control_x1 : Int
            First x-coordinate of the control point.

        Examples
        --------
        >>> import apysc as ap
        >>> bezier_3d: ap.PathBezier3D = ap.PathBezier3D(
        ...     control_x1=50,
        ...     control_y1=0,
        ...     control_x2=100,
        ...     control_y2=0,
        ...     dest_x=150,
        ...     dest_y=50,
        ... )
        >>> bezier_3d.control_x1 = ap.Int(75)
        >>> bezier_3d.control_x1
        Int(75)
        """
        self._initialize_control_x1_if_not_initialized()
        return self._control_x1._copy()

    @control_x1.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def control_x1(self, value: Int) -> None:
        """
        Set a first x-coordinate of the control point.

        Parameters
        ----------
        value : Int
            First x-coordinate of the control point.
        """
        self._initialize_control_x1_if_not_initialized()
        self._control_x1.value = value

        self._append_control_x1_linking_setting()

    _control_x1_snapshots: Dict[str, int]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_control_x1_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_control_x1_snapshots",
            value=int(self._control_x1._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._initialize_control_x1_if_not_initialized()
        self._control_x1._value = self._control_x1_snapshots[snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: return.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Return class": "# Return クラス",
    ##################################################
    "This page explains the `Return` class.": "このページでは`Return`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page (the apysc uses the `Return` class for the same reason of each apysc data type):": "このページを読み進める前に以下のページを確認しておくと役に立つかもしれません（apyscライブラリでは`Return`クラスを各データクラスと同じような理由で使用しています）。",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the Return class?": "## Return クラスの概要",
    ##################################################
    "The `Return` class behaves to append the `return;` JavaScript code. Therefore, this class can be used only in an event handler (function or method) scope.": "`Return`クラスはJavaScriptの`return;`のコードのように振る舞います。従って、このクラスはイベントハンドラの関数もしくはメソッド内のスコープでのみ使用することができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Return` class constructor accepts no arguments. You can use this interface with the branch condition, for example, the `ap.If` class.": "`Return`クラスのコンストラクタは引数を必要としません。このクラスは`ap.If`などの条件分岐の記述内などで使うことができます。",  # noqa
    ##################################################
    "The following example changes the rectangle fill color when you click it. Each `ap.If` branch instantiate `Return` class, so the code applies the changing of fill color one by one:": "以下のコード例では四角をクリックするたびに塗りの色を変更しています。各`ap.If`での分岐内では`Return`クラスを使用しているため、1回のクリックでは1回の色変更のみ行われます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    fill_color: ap.String = rectangle.fill_color\n    with ap.If(fill_color == "#00aaff"):\n        rectangle.fill_color = ap.String("#ff00aa")\n        ap.Return()\n    with ap.If(fill_color == "#ff00aa"):\n        rectangle.fill_color = ap.String("#00ffaa")\n        ap.Return()\n    with ap.If(fill_color == "#00ffaa"):\n        rectangle.fill_color = ap.String("#00aaff")\n        ap.Return()\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="return_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    fill_color: ap.String = rectangle.fill_color\n    with ap.If(fill_color == "#00aaff"):\n        rectangle.fill_color = ap.String("#ff00aa")\n        ap.Return()\n    with ap.If(fill_color == "#ff00aa"):\n        rectangle.fill_color = ap.String("#00ffaa")\n        ap.Return()\n    with ap.If(fill_color == "#00ffaa"):\n        rectangle.fill_color = ap.String("#00aaff")\n        ap.Return()\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="return_basic_usage/")\n```',  # noqa
    ##################################################
    "## Return API": "## Return API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Class for the return expression.<hr>": "return のコード表現のためのクラスです。<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This class can be instantiated only in an event handler scope.<hr>": "このクラスはイベントハンドラのスコープ内でのみインスタンス化することができます。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     '''\n...     The handler that the timer calls.\n...\n```": "```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     '''\n...     The handler that the timer calls.\n...\n```",  # noqa
}



"""Implementation of the VariableNameMixIn class.
"""

from typing import List

from typing_extensions import final

from apysc._type.deleted_object_mixin import DeletedObjectMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class VariableNameMixIn(DeletedObjectMixIn):

    _variable_name: str
    _variable_name_history: List[str]

    @property
    def variable_name(self) -> str:
        """
        Get a js variable name of this instance.

        Returns
        -------
        variable_name : str
            A js variable name of this instance.
        """
        if not hasattr(self, "_variable_name"):
            return ""
        variable_name: str = self._variable_name
        if isinstance(self, VariableNameSuffixMixIn):
            suffix: str = self._variable_name_suffix
            if suffix != "" and not variable_name.endswith(f"__{suffix}"):
                variable_name += f"__{suffix}"
        return variable_name

    @variable_name.setter
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    @arg_validation_decos.not_empty_string(arg_position_index=1)
    def variable_name(self, variable_name: str) -> None:
        """
        Set a js variable name of this instance.

        Parameters
        ----------
        variable_name : str
            Variable name to set.
        """
        if not hasattr(self, "_variable_name_history"):
            self._variable_name_history = []
        self._variable_name = variable_name
        self._variable_name_history.append(variable_name)

    @final
    def _get_previous_variable_name(self) -> str:
        """
        Get a previous variable name.

        Returns
        -------
        previous_variable_name : str
            A previous-variable name of this instance.
            If that value is not existing, this interface
            returns a blank string.
        """
        if not hasattr(self, "_variable_name_history"):
            return ""
        if len(self._variable_name_history) <= 1:
            return ""
        return self._variable_name_history[-2]



"""Common JavaScript library utility implementations.

Mainly the following interfaces are defined:

- get_jslib_file_names
    Get the JavaScript libraries file's names.
- get_jslib_abs_dir_path
    Get the Javascript library's absolute directory path.
    This interface returns this module's directory.
- export_jslib_to_specified_dir
    Export a JavaScript library to a specified directory.
- read_jslib_str
    Read a JavaScript library file str.
"""

import os
import re
import shutil
import sys
from types import ModuleType
from typing import List
from typing import Match
from typing import Optional


def get_jslib_file_names() -> List[str]:
    """
    Get the JavaScript libraries file's names.

    Returns
    -------
    jslib_file_names : list of str
        JavaScript libraries file names existing in this module's
        directory.
        e.g., ['jquery-3.5.1.min.js', 'svg-3.1.2.min.js']
    """
    this_modules_dir_path: str = get_jslib_abs_dir_path()
    jslib_file_names: List[str] = []
    file_names: List[str] = os.listdir(this_modules_dir_path)
    for file_name in file_names:
        if not file_name.endswith(".js"):
            continue
        jslib_file_names.append(file_name)
    jslib_file_names = _sort_js_file_names_with_priority_setting(
        jslib_file_names=jslib_file_names
    )
    return jslib_file_names


def _sort_js_file_names_with_priority_setting(
    *, jslib_file_names: List[str]
) -> List[str]:
    """
    Sort a JavaScript libraries' file names list
    with the priority setting.

    Parameters
    ----------
    jslib_file_names : List[str]
        A target list to sort.

    Returns
    -------
    sorted_jslib_file_names : List[str]
        A sorted list.
    """
    jslib_file_names.sort()
    sorted_jslib_file_names: List[str] = _HIGH_PRIORITY_FILE_ORDERS.copy()
    for jslib_file_name in jslib_file_names:
        if jslib_file_name in _HIGH_PRIORITY_FILE_ORDERS:
            continue
        sorted_jslib_file_names.append(jslib_file_name)
    return sorted_jslib_file_names


def get_jslib_abs_dir_path() -> str:
    """
    Get the Javascript library's absolute directory path.
    This interface returns this module's directory.

    Returns
    -------
    jslib_abs_dir_path : str
        Javascript library's absolute directory path.
        This module's directory will be set.
    """
    this_module: ModuleType = sys.modules[__name__]
    jslib_abs_dir_path: str = os.path.dirname(str(this_module.__file__))
    return jslib_abs_dir_path


def get_jquery_file_name() -> str:
    """
    Get the jQuery file name.

    Returns
    -------
    file_name : str
        The jQuery file name.

    Raises
    ------
    FileNotFoundError
        If the file does not exist.
    """
    this_modules_dir_path: str = get_jslib_abs_dir_path()
    jslib_file_names: List[str] = []
    file_names: List[str] = os.listdir(this_modules_dir_path)
    for file_name in file_names:
        if not file_name.endswith(".js"):
            continue
        jslib_file_names.append(file_name)
    for jslib_name in jslib_file_names:
        match: Optional[Match] = re.search(
            pattern=r"jquery-\d+?\.\d+?\.\d+?\.min\.js", string=jslib_name
        )
        if match is None:
            continue
        return jslib_name
    raise FileNotFoundError("The jQuery file does not exist in the _jslib package.")


_HIGH_PRIORITY_FILE_ORDERS: List[str] = [
    get_jquery_file_name(),
]


def export_jslib_to_specified_dir(*, dest_dir_path: str, jslib_name: str) -> str:
    """
    Export a JavaScript library to a specified directory.

    Parameters
    ----------
    dest_dir_path : str
        Directory path to export JavaScript library file.
    jslib_name : str
        JavaScript file name to export.

    Returns
    -------
    dest_file_path : str
        Exported Javascript library's file path.

    Raises
    ------
    FileNotFoundError
        If there is no specified JavaScript file.
    """
    os.makedirs(dest_dir_path, exist_ok=True)
    dir_path: str = get_jslib_abs_dir_path()
    src_file_path: str = os.path.join(dir_path, jslib_name)
    if not os.path.isfile(src_file_path):
        raise FileNotFoundError(
            "Specified JavaScript library file is not found: " f"{src_file_path}"
        )
    dest_file_path: str = os.path.join(dest_dir_path, jslib_name)
    shutil.copyfile(src_file_path, dest_file_path)
    return dest_file_path


def read_jslib_str(*, jslib_name: str) -> str:
    """
    Read a JavaScript library file str.

    Parameters
    ----------
    jslib_name : str
        JavaScript file name to read.

    Returns
    -------
    jslib_str : str
        Read JavaScript library string.
    """
    from apysc._file import file_util

    dir_path: str = get_jslib_abs_dir_path()
    file_path: str = os.path.join(dir_path, jslib_name)
    jslib_str: str = file_util.read_txt(file_path=file_path)
    return jslib_str



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_path.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_path interface": "# Graphics クラスの draw_path インターフェイス",
    ##################################################
    "This page explains the `Graphics` class `draw_path` interface.": "このページでは`Graphics`クラスの`draw_path`インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `draw_path` interface draws vector graphics of a path.": "`draw_path`インターフェイスはベクターグラフィックスのパスを描画します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `draw_path` interface requires the `path_data_list` argument.": "`draw_path`インターフェイスは`path_data_list`引数の指定を必要とします。",  # noqa
    ##################################################
    "The `path_data_list` argument is a list of path data, such as the `PathLineTo` or `PathBezier2D`.": "`path_data_list`引数は`PathLineTo`や`PathBezier2D`などのパスデータの配列となります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=300, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color="#0af", thickness=5)\npath: ap.Path = sprite.graphics.draw_path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=100),\n        ap.PathLineTo(x=100, y=100),\n        ap.PathLineTo(x=150, y=50),\n        ap.PathBezier2D(\n            control_x=200,\n            control_y=100,\n            dest_x=250,\n            dest_y=50,\n        ),\n    ],\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_path_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=300, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color="#0af", thickness=5)\npath: ap.Path = sprite.graphics.draw_path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=100),\n        ap.PathLineTo(x=100, y=100),\n        ap.PathLineTo(x=150, y=50),\n        ap.PathBezier2D(\n            control_x=200,\n            control_y=100,\n            dest_x=250,\n            dest_y=50,\n        ),\n    ],\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_path_basic_usage/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Path class](path.md)": "- [Path クラス](jp_path.md)",
    ##################################################
    "- [PathMoveTo class](path_move_to.md)": "- [PathMoveTo クラス](jp_path_move_to.md)",
    ##################################################
    "- [PathLineTo class](path_line_to.md)": "- [PathLineTo クラス](jp_path_line_to.md)",
    ##################################################
    "- [PathHorizontal class](path_horizontal.md)": "- [PathHorizontal クラス](jp_path_horizontal.md)",  # noqa
    ##################################################
    "- [PathVertical class](path_vertical.md)": "- [PathVertical クラス](jp_path_vertical.md)",  # noqa
    ##################################################
    "- [PathClose class](path_close.md)": "- [PathClose クラス](jp_path_close.md)",
    ##################################################
    "- [PathBezier2D class](path_bezier_2d.md)": "- [PathBezier2D クラス](jp_path_bezier_2d.md)",  # noqa
    ##################################################
    "- [PathBezier2DContinual class](path_bezier_3d.md)": "- [PathBezier2DContinual クラス](jp_path_bezier_3d.md)",  # noqa
    ##################################################
    "- [PathBezier3DContinual class](path_bezier_3d_continual.md)": "- [PathBezier3DContinual クラス](jp_path_bezier_3d_continual.md)",  # noqa
    ##################################################
    "## draw_path API": "## draw_path API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a path vector graphics.<hr>": "パスのベクターグラフィックスを描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `path_data_list`: list of PathDataBase": "- `path_data_list`: list of PathDataBase",  # noqa
    ##################################################
    "  - Target path data settings, such as the ap.PathData.MoveTo.": "  - ap.PathData.MoveToなどの対象のパスデータの設定のリスト。",  # noqa
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `path`: Path": "- `path`: Path",
    ##################################################
    "  - Created path graphics instance.": "  - 作成されたパスのグラフィックスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color="#fff", thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=50),\n...         ap.PathBezier2D(control_x=50, control_y=0, dest_x=100, dest_y=50),\n...     ]\n... )\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color="#fff", thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=50),\n...         ap.PathBezier2D(control_x=50, control_y=0, dest_x=100, dest_y=50),\n...     ]\n... )\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Path class](https://simon-ritchie.github.io/apysc/en/path.html)": "- [Path クラス](https://simon-ritchie.github.io/apysc/jp/jp_path.html)",  # noqa
    ##################################################
    "- [PathMoveTo class](https://simon-ritchie.github.io/apysc/en/path_move_to.html)": "- [PathMoveTo クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_move_to.html)",  # noqa
    ##################################################
    "- [PathLineTo class](https://simon-ritchie.github.io/apysc/en/path_line_to.html)": "- [PathLineTo クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_line_to.html)",  # noqa
    ##################################################
    "- [PathHorizontal class](https://simon-ritchie.github.io/apysc/en/path_horizontal.html)": "- [PathHorizontal クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_horizontal.html)",  # noqa
    ##################################################
    "- [PathVertical class](https://simon-ritchie.github.io/apysc/en/path_vertical.html)": "- [PathVertical クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_vertical.html)",  # noqa
    ##################################################
    "- [PathClose class](https://simon-ritchie.github.io/apysc/en/path_close.html)": "- [PathClose クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_close.html)",  # noqa
    ##################################################
    "- [PathBezier2D class](https://simon-ritchie.github.io/apysc/en/path_bezier_2d.html)": "- [PathBezier2D クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_bezier_2d.html)",  # noqa
    ##################################################
    "- [PathBezier2DContinual class](https://simon-ritchie.github.io/apysc/en/path_bezier_2d_continual.html)": "- [PathBezier2DContinual クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_bezier_2d_continual.html)",  # noqa
    ##################################################
    "- [PathBezier3D class](https://simon-ritchie.github.io/apysc/en/path_bezier_3d.html)": "- [PathBezier3D クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_bezier_3d.html)",  # noqa
    ##################################################
    "- [PathBezier3DContinual class](https://simon-ritchie.github.io/apysc/en/path_bezier_3d_continual.html)": "- [PathBezier3DContinual クラス](https://simon-ritchie.github.io/apysc/jp/jp_path_bezier_3d_continual.html)",  # noqa
}



"""This module is for the geometry-related values' validation
helper interfaces.
"""


from typing import Any


def validate_point_2d_type(*, point: Any, additional_err_msg: str = "") -> None:
    """
    Validate specified value's type is Point2D.

    Parameters
    ----------
    point : Point2D
        Point2D instance to be checked.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value's type is not Point2D.
    """
    import apysc as ap

    if isinstance(point, ap.Point2D):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        f"Specified value's type is not Point2D: {type(point)}" f"{additional_err_msg}"
    )



"""Class implementations for the minute-related mix-in.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class MinuteMixIn(VariableNameMixIn, VariableNameSuffixAttrMixIn, RevertMixIn):

    _initial_minute: Union[int, Int]
    _minute: Int

    @final
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_minute_int(arg_position_index=1)
    def _set_init_minute_value(self, *, minute: Union[int, Int]) -> None:
        """
        Set an initial minute value.

        Parameters
        ----------
        minute : Union[int, Int]
            A minute value to set.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_int_from_builtin_val,
        )

        self._initial_minute = minute
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="minute")
        self._minute = get_copied_int_from_builtin_val(
            integer=minute, variable_name_suffix=suffix
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _get_init_minute_argument_expression(self) -> str:
        """
        Get an initial minute's argument expression string.

        Returns
        -------
        expression : str
            A created expression string.
        """
        if isinstance(self._initial_minute, Int):
            return f", {self._initial_minute.variable_name}"
        return f", {self._minute._value}"

    @property
    @add_debug_info_setting(module_name=__name__)
    def minute(self) -> Int:
        """
        Get a current minute's value.

        Returns
        -------
        minute : Int
            A current minute value.

        References
        ----------
        - DateTime class minute property
            - https://simon-ritchie.github.io/apysc/en/datetime_minute.html

        Examples
        --------
        >>> import apysc as ap
        >>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1, minute=30)
        >>> datetime_.minute
        Int(30)
        >>> datetime_.minute = ap.Int(50)
        >>> datetime_.minute
        Int(50)
        """
        copied_minute_val: Int = self._minute._copy()
        self._append_minute_getter_expression(minute_val=copied_minute_val)
        return copied_minute_val

    @minute.setter
    @arg_validation_decos.is_minute_int(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def minute(self, value: Int) -> None:
        """
        Set a minute value.

        Parameters
        ----------
        value : Int
            A minute value to set.

        References
        ----------
        - DateTime class minute property
            - https://simon-ritchie.github.io/apysc/en/datetime_minute.html

        Examples
        --------
        >>> import apysc as ap
        >>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=1, minute=30)
        >>> datetime_.minute
        Int(30)
        >>> datetime_.minute = ap.Int(50)
        >>> datetime_.minute
        Int(50)
        """
        self._minute = value._copy()
        self._append_minute_setter_expression(minute_val=value)

    @final
    @arg_validation_decos.is_minute_int(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def _append_minute_getter_expression(self, *, minute_val: Int) -> None:
        """
        Append a minute's getter expression string.

        Parameters
        ----------
        minute_val : Int
            A month value to use in an expression.
        """
        import apysc as ap

        expression: str = (
            f"{minute_val.variable_name} = {self.variable_name}.getMinutes();"
        )
        ap.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_minute_int(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def _append_minute_setter_expression(self, *, minute_val: Int) -> None:
        """
        Append a minute's setter expression string.

        Parameters
        ----------
        minute_val : Int
            A minute value to use in an expression.
        """
        import apysc as ap

        expression: str = (
            f"{self.variable_name}.setMinutes({minute_val.variable_name});"
        )
        ap.append_js_expression(expression=expression)

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._minute._run_all_make_snapshot_methods(snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._minute._run_all_revert_methods(snapshot_name=snapshot_name)



"""Number validation implementations.

Mainly following interfaces are defined:

- validate_num
    - Validate a specified value as an integer or float type.
- validate_integer
    - Validate whether a specified value is an integer or not.
- validate_builtin_integer
    - Validate whether a specified value is a built-in integer or not.
- validate_int_is_zero_or_one
    - Validate specified integer value is zero or one.
- validate_num_is_gt_zero
    - Validate specified value is greater than zero.
- validate_num_is_gte_zero
    - Validate whether a specified value is greater than or
    equal to zero.
- validate_num_is_0_to_1_range
    - Validate whether a specified number is from 0.0 to 1.0.
"""

from typing import Union

from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.number_value_mixin import NumberValueMixIn


def validate_num(
    *, num: Union[int, float, Int, Number], additional_err_msg: str = ""
) -> None:
    """
    Validate a specified value as an integer or float type.

    Parameters
    ----------
    num : int or float or Int or Number
        Number value to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value is not an integer and float value.
    """
    if isinstance(num, (int, float, NumberValueMixIn)):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        f"Specified value is not iteger or float type: {num}"
        f"({type(num)}){additional_err_msg}"
    )


def validate_integer(*, integer: Union[int, Int], additional_err_msg: str = "") -> None:
    """
    Validate whether a specified value is an integer or not.

    Parameters
    ----------
    integer : Int or int
        Integer value to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value is not an integer.
    """
    if isinstance(integer, (int, Int)):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        f"A specified value is not an integer: {integer}({type(integer)})"
        f"{additional_err_msg}"
    )


def validate_builtin_integer(*, integer: int, additional_err_msg: str = "") -> None:
    """
    Validate whether a specified value is a built-in integer or not.

    Parameters
    ----------
    integer : int
        A target integer to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value is not a built-in integer.
    """
    if isinstance(integer, int):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        "A specified value is not a built-in integer: "
        f"{integer}({type(integer)})"
        f"{additional_err_msg}"
    )


def validate_int_is_zero_or_one(*, integer: Union[int, Int]) -> None:
    """
    Validate specified integer value is zero or one.

    Notes
    -----
    This interface skips validation if an argument
    value is not an Int or int instance.

    Parameters
    ----------
    integer : Int or int
        Integer value to check.

    Raises
    ------
    ValueError
        If a specified integer is not zero and one.
    """
    if not isinstance(integer, (int, Int)):
        return
    if isinstance(integer, int):
        if integer == 0 or integer == 1:
            return
    elif isinstance(integer, Int):
        if integer._value == 0 or integer._value == 1:
            return
    raise ValueError(f"Specified integer value is not zero and one: {integer}")


def validate_num_is_gt_zero(
    *, num: Union[int, float, Int, Number], additional_err_msg: str = ""
) -> None:
    """
    Validate specified value is greater than zero.

    Parameters
    ----------
    num : int or float or Int or Number
        Number value to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value is less than or equal to zero.
    """
    if isinstance(num, NumberValueMixIn):
        if num._value > 0:
            return
    elif num > 0:
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        f"Specified values is less than or equal to zero: {num}" f"{additional_err_msg}"
    )


def validate_num_is_gte_zero(
    *, num: Union[int, float, Int, Number], additional_err_msg: str = ""
) -> None:
    """
    Validate whether a specified value is greater than or equal to zero.

    Parameters
    ----------
    num : int or float or Int or Number
        Number value to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value is less than zero.
    """
    if isinstance(num, NumberValueMixIn):
        if num._value >= 0:
            return
    elif num >= 0:
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(f"Specified values is less than zero: {num}{additional_err_msg}")


def validate_num_is_0_to_1_range(
    *, num: Union[float, NumberValueMixIn], additional_err_msg: str = ""
) -> None:
    """
    Validate whether a specified number is from 0.0 to 1.0.

    Parameters
    ----------
    num : float or Number
        A number value to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified opacity is out of the 0.0 to 1.0 range.
    """
    lt_err_msg: str = "Can't specify number value less than 0.0: "
    gt_err_msg: str = "Can't specify number value greater than 1.0: "
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    if isinstance(num, NumberValueMixIn):
        if num._value < 0.0:
            raise ValueError(f"{lt_err_msg}{num}{additional_err_msg}")
        if num._value > 1.0:
            raise ValueError(f"{gt_err_msg}{num}{additional_err_msg}")
    else:
        if num < 0.0:
            raise ValueError(f"{lt_err_msg}{num}{additional_err_msg}")
        if num > 1.0:
            raise ValueError(f"{gt_err_msg}{num}{additional_err_msg}")



"""This module is for the translation mapping data of the
following document:

Document file: mouse_event_abstract.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# MouseEvent interfaces abstract": "# MouseEvent の各インターフェイスの概要",
    ##################################################
    "This page explains the MouseEvent interfaces abstract.": "このページでは MouseEvent クラスの各インターフェイスの概要について説明します。",  # noqa
    ##################################################
    "## What apysc can do in its interfaces": "## これらの各インターフェイスでapyscが出来ること",
    ##################################################
    "- You can set the `MouseEvent` handlers, such as the click, mouse down, mouse over, and so on, to each graphic instance.": "- クリックやマウスダウン、マウスオーバーなどの`MouseEvent`の各ハンドラをグラフィックスのインスタンスへ設定することができます。",  # noqa
    ##################################################
    "- You can pass the optional arguments to the handler.": "- ハンドラの引数へ任意のパラメーターを渡すことができます。",  # noqa
    ##################################################
    "## Example of the click event": "## クリックイベントの例",
    ##################################################
    "To bind MouseEvent, defining the handler function (or method) would be necessary (e.g., `on_click`).": "マウスイベントを設定するにはまずはハンドラ用の関数（もしくはメソッド）の定義が必要になります（例: `on_click`）。",  # noqa
    ##################################################
    "These handlers can bind with the click interface.": "これらのハンドラは click のインターフェイスで登録することができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    with ap.If(rectangle.fill_color == "#00aaff"):\n        rectangle.fill_color = ap.String("#f0a")\n        ap.Return()\n\n    with ap.If(rectangle.fill_color == "#ff00aa"):\n        rectangle.fill_color = ap.String("#0af")\n        ap.Return()\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="mouse_event_abstract_click/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    with ap.If(rectangle.fill_color == "#00aaff"):\n        rectangle.fill_color = ap.String("#f0a")\n        ap.Return()\n\n    with ap.If(rectangle.fill_color == "#ff00aa"):\n        rectangle.fill_color = ap.String("#0af")\n        ap.Return()\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="mouse_event_abstract_click/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "There are a lot of other mouse event binding interfaces, such as the mouse down, mouse over, and mouse move. For more details, please see the following:": "他にもマウスダウンやマウスオーバー、マウスムーブなど様々なイベント設定用のインターフェイスが存在します。詳細は以下をご確認ください:",  # noqa
    ##################################################
    "- [Basic mouse event interfaces](mouse_event_basic.md)": "- [基本的なマウスイベントの各インターフェイス](jp_mouse_event_basic.md)",  # noqa
    ##################################################
    "- [click interface](click.md)": "- [click インターフェイス](jp_click.md)",
    ##################################################
    "- [dblclick interface](dblclick.md)": "- [dblclick インターフェイス](jp_dblclick.md)",
    ##################################################
    "- [mousedown and mouseup interfaces](mousedown_and_mouseup.md)": "- [mousedown と mouseup のインターフェイス](jp_mousedown_and_mouseup.md)",  # noqa
    ##################################################
    "- [mouseover and mouseout interfaces](mouseover_and_mouseout.md)": "- [mouseover と mouseout のインターフェイス](jp_mouseover_and_mouseout.md)",  # noqa
    ##################################################
    "- [mousemove interface](mousemove.md)": "- [mousemove インターフェイス](jp_mousemove.md)",
}



"""This module is for the translation mapping data of the
following document:

Document file: add_child_and_remove_child.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# add_child and remove_child interfaces": "# add_child と remove_child インターフェイス",
    ##################################################
    "This page explains the container class, like the `Graphics`\\, `Sprite`\\, `Stage`) `add_child` and `remove_child` method interfaces.": "このページではGraphicsクラスやSprite、Stageクラスなどのコンテナーとして扱えるクラスが持つ`add_child`と`remove_child`のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `add_child` and `remove_child` add or remove a `DisplayObject` child instance from a container instance. The apysc does not display a removed `DisplayObject` instance.": "`add_child`インターフェイスではコンテナーのインスタンスへ子となる各`DisplayObject`を継承したインスタンスを追加し、逆に`remove_child`インターフェイスでは子のインスタンスをコンテナーから取り除きます。apyscでは取り除かれた子のインスタンスは表示されなくなります。",  # noqa
    ##################################################
    "## Automatic addition of the children": "## 子のインスタンスの自動追加について",
    ##################################################
    "The apysc appends each `DisplayObject` instance to a parent at the constructor. So, for example, it appends a `Sprite` instance to a parent stage. Similarly, it appends an instance of a graphic to a parent `Sprite` instance.": "apyscでは各`DisplayObject`のインスタンスはコンストラクタの時点で親のインスタンスへと自動で追加されます。例えば`Sprite`クラスであれば`Stage`クラスのインスタンスを親として追加され、`Sprite`クラスを親として内部で作成される`graphics`プロパティのインスタンスは`Sprite`クラスのインスタンスへと自動で追加されます。",  # noqa
    ##################################################
    "If you need to adjust a parent, it is necessary to call the `add_child` or `remove_child` interfaces manually (for instance, set a `Sprite` parent to the other `Sprite`).": "もし親のインスタンスを調整したい場合には手動で`add_child`や`remove_child`などのインターフェイスを呼ぶ必要があります。例えば親としてのとある`Sprite`クラスのインスタンスから別の`Sprite`クラスのインスタンスに子を移したい場合などが該当します。",  # noqa
    ##################################################
    "## Basic usage of the remove_child interface": "## remove_child インターフェイスの基本的な使い方",
    ##################################################
    "The `remove_child` interface removes a child from a parent container instance. The apysc does not display a removed `DisplayObject` instance.": "`remove_child`インターフェイスでは子を親のコンテナー要素から取り除きます。apyscは取り除かれた`DisplayObject`の子のインスタンスを表示しません。",  # noqa
    ##################################################
    "For example, the following code calls the `remove_child` interface in the click handler, so if you click the rectangle, it removes that rectangle.": "例えば以下のコードでは四角をクリックした際のハンドラ内で`remove_child`インターフェイスを呼び出しており、クリック時に四角が画面から取り除かれます。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle: ap.Rectangle = options["rectangle"]\n    sprite.remove_child(child=rectangle)\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="sprite_basic_usage_of_remove_child/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle: ap.Rectangle = options["rectangle"]\n    sprite.remove_child(child=rectangle)\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="sprite_basic_usage_of_remove_child/")\n```',  # noqa
    ##################################################
    "## The basic usage of the add_child interface": "## add_child インターフェイスの基本的な使い方",
    ##################################################
    "The `add_child` interface adds a removed child again or adds a child to the other container instance.": "`add_child`インターフェイスは取り除かれた子のインスタンスをもう一度他の親のコンテナーのインスタンスへと追加します。",  # noqa
    ##################################################
    "The following code example removes the rectangle from the first `Sprite` container (be positioned to the left) when you click the rectangle. Also, that click event adds the rectangle to the second `Sprite` container (be positioned to the right).": "以下のコードでは四角をクリックした際に1つ目の左に配置されている`Sprite`の親のコンテナーからその四角を取り除き、そして2つ目の右側に配置してある`Sprite`のインスタンスへと子を追加しています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _SpriteAndRectOptions(TypedDict):\n    rectangle: ap.Rectangle\n    sprite: ap.Sprite\n\n\ndef on_sprite_click(\n    e: ap.MouseEvent[ap.Sprite], options: _SpriteAndRectOptions\n) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    first_sprite: ap.Sprite = e.this\n    rectangle: ap.Rectangle = options["rectangle"]\n    second_sprite: ap.Sprite = options["sprite"]\n    first_sprite.remove_child(child=rectangle)\n    second_sprite.add_child(child=rectangle)\n\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\n\nfirst_sprite: ap.Sprite = ap.Sprite()\nfirst_sprite.graphics.begin_fill(color="#0af")\nfirst_sprite.x = ap.Int(50)\nfirst_sprite.y = ap.Int(50)\nrectangle: ap.Rectangle = first_sprite.graphics.draw_rect(x=0, y=0, width=50, height=50)\n\nsecond_sprite: ap.Sprite = ap.Sprite()\nsecond_sprite.x = ap.Int(150)\nsecond_sprite.y = ap.Int(50)\n\noptions: _SpriteAndRectOptions = {"rectangle": rectangle, "sprite": second_sprite}\nfirst_sprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="sprite_basic_usage_of_add_child/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _SpriteAndRectOptions(TypedDict):\n    rectangle: ap.Rectangle\n    sprite: ap.Sprite\n\n\ndef on_sprite_click(\n    e: ap.MouseEvent[ap.Sprite], options: _SpriteAndRectOptions\n) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    first_sprite: ap.Sprite = e.this\n    rectangle: ap.Rectangle = options["rectangle"]\n    second_sprite: ap.Sprite = options["sprite"]\n    first_sprite.remove_child(child=rectangle)\n    second_sprite.add_child(child=rectangle)\n\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\n\nfirst_sprite: ap.Sprite = ap.Sprite()\nfirst_sprite.graphics.begin_fill(color="#0af")\nfirst_sprite.x = ap.Int(50)\nfirst_sprite.y = ap.Int(50)\nrectangle: ap.Rectangle = first_sprite.graphics.draw_rect(x=0, y=0, width=50, height=50)\n\nsecond_sprite: ap.Sprite = ap.Sprite()\nsecond_sprite.x = ap.Int(150)\nsecond_sprite.y = ap.Int(50)\n\noptions: _SpriteAndRectOptions = {"rectangle": rectangle, "sprite": second_sprite}\nfirst_sprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="sprite_basic_usage_of_add_child/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [DisplayObject class parent interfaces](display_object_parent.md)": "- [DisplayObjectクラス parent （親要素属性）のインターフェイス](jp_display_object_parent.md)",  # noqa
    ##################################################
    "- [contains interface](contains.md)": "- [contains インターフェイス](jp_contains.md)",
    ##################################################
    "## add_child API": "## add_child API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add display object child to this instance.<hr>": "表示オブジェクトの子をこのインスタンスへと追加します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `child`: DisplayObject": "- `child`: DisplayObject",
    ##################################################
    "  - Child instance to add.": "  - 追加する子のインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite_1: ap.Sprite = ap.Sprite()\n>>> sprite_1.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite_1.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> sprite_2: ap.Sprite = ap.Sprite()\n>>> sprite_2.add_child(rectangle)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite_1: ap.Sprite = ap.Sprite()\n>>> sprite_1.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite_1.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> sprite_2: ap.Sprite = ap.Sprite()\n>>> sprite_2.add_child(rectangle)\n```',  # noqa
    ##################################################
    "## remove_child API": "## remove_child API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Remove display object child from this instance.<hr>": "このインスタンスから指定された表示オブジェクトの子を取り除きます。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `child`: DisplayObject": "- `child`: DisplayObject",
    ##################################################
    "  - Child instance to remove.": "  - 取り除く対象の子のインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> sprite.graphics.remove_child(rectangle)\n>>> print(rectangle.parent)\nNone\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> sprite.graphics.remove_child(rectangle)\n>>> print(rectangle.parent)\nNone\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_line.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_line interface": "# Graphics クラスの draw_line インターフェイス",
    ##################################################
    "This page explains the `Graphics` class `draw_line` method interface.": "このページでは`Graphics`クラスの`draw_line`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "`draw_line` interface will draw the simple straight line graphics. This interface will ignore `dot_setting`, `dash_setting`, `round_dot_setting`, and `dash_dot_setting`.": "`draw_line`インターフェイスはシンプルな直線のグラフィックスを描画します。このインターフェイスは`dot_setting`、`dash_setting`、`round_dot_setting`、`dash_dot_setting`などの引数や属性の設定を無視します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "`draw_line` inteface has `x_start` (line x-start coordinate), `y_start` (line y-start coordinate), `x_end` (line x-end coordinate), and `y_end` (line y-end coordinate) arguments.": "`draw_line`インターフェイスは`x_start`（線の開始位置のX座標）、`y_start`（線の開始位置のY座標）、`x_end`（線の終了位置のX座標）、`y_end`（線の終了位置のY座標）の各引数を必要とします。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color="#0af", thickness=5)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_line_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color="#0af", thickness=5)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_line_basic_usage/")\n```',  # noqa
    ##################################################
    "## Ignored line style settings": "## 無視される線のスタイル設定",
    ##################################################
    "This interface will ignore `dot_setting`, `dash_setting`, `round_dot_setting`, and `dash_dot_setting` for simplicity. If you need to draw these styled lines, then use `draw_dotted_line`, `draw_dashed_line`, `draw_round_dotted_line`, or `draw_dash_dotted_line` interfaces instead of the `draw_line` interface.": "このインターフェイスはインターフェイスのシンプルさのために`dot_setting`、`dash_setting`、`round_dot_setting`、`dash_dot_setting`の各設定を無視します。もしもこれらのスタイル設定が必要な場合には`draw_line`インターフェイスの代わりに`draw_dotted_line`、`draw_dashed_line`、`draw_round_dotted_line`、`draw_dash_dotted_line`などのインターフェイスを仕様してください。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# dot_setting will be ignored, and the result line will not be dotted.\nsprite.graphics.line_style(\n    color="#0af", thickness=5, dot_setting=ap.LineDotSetting(dot_size=5)\n)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_line_ignored_dot_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# dot_setting will be ignored, and the result line will not be dotted.\nsprite.graphics.line_style(\n    color="#0af", thickness=5, dot_setting=ap.LineDotSetting(dot_size=5)\n)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_line_ignored_dot_setting/")\n```',  # noqa
    ##################################################
    "## Line class instance": "## Line クラスのインスタンス",
    ##################################################
    "`draw_line` interface returns the `Line` instance. You can update each setting or bind events to that instance. `draw_dotted_line`, `draw_dashed_line`, `draw_round_dotted_line`\n, and `draw_dash_dotted_line` will also return the same type instance.": "`draw_line`インターフェイスは`Line`クラスのインスタンスを返却します。そのインスタンスの各種設定を変更したりイベントを登録したり等を行うことができます。`draw_dotted_line`、`draw_dashed_line`、`draw_round_dotted_line`、`draw_dash_dotted_line`などのインターフェイスも同じく`Line`クラスのインスタンスを返却します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color="#0af", thickness=5)\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\n# Update the line color from cyan to magenta.\nline.line_color = ap.String("#f0a")\n\nap.save_overall_html(dest_dir_path="graphics_draw_line_line_instance/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color="#0af", thickness=5)\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\n# Update the line color from cyan to magenta.\nline.line_color = ap.String("#f0a")\n\nap.save_overall_html(dest_dir_path="graphics_draw_line_line_instance/")\n```',  # noqa
    ##################################################
    "## draw_line API": "## draw_line API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a normal line vector graphic.<hr>": "通常の直線のベクターグラフィックスを描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x_start`: Int or int": "- `x_start`: Int or int",
    ##################################################
    "  - Line start x-coordinate.": "  - 線の開始位置のX座標。",
    ##################################################
    "- `y_start`: Int or int": "- `y_start`: Int or int",
    ##################################################
    "  - Line start y-coordinate.": "  - 線の開始位置のY座標。",
    ##################################################
    "- `x_end`: Int or int": "- `x_end`: Int or int",
    ##################################################
    "  - Line end x-coordinate.": "  - 線の終了位置のX座標。",
    ##################################################
    "- `y_end`: Int or int": "- `y_end`: Int or int",
    ##################################################
    "  - Line end y-coordinate.": "  - 線の終了位置のY座標。",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line`: Line": "- `line`: Line",
    ##################################################
    "  - Created line graphics instance.": "  - 生成された線のグラフィックスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    " ・This interface ignores line settings, like the `LineDotSetting`, `LineDashSetting`.<hr>": " ・このインターフェイスは`LineDotSetting`や`LineDashSetting`などの設定を無視します。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\"#fff\", thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_color\nString('#ffffff')\n\n>>> line.line_thickness\nInt(5)\n```": "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\"#fff\", thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_color\nString('#ffffff')\n\n>>> line.line_thickness\nInt(5)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: string.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class": "# String クラス",
    ##################################################
    "This page explains the `String` class.": "このページでは`String`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page:": "事前に以下のページを確認しておくと読み進める上で役に立つかもしれません:",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the String class?": "## String クラスの概要",
    ##################################################
    "The `String` class is the apysc string class. It can accept `str` or `String` values at the constructor, as follows:": "`String`クラスはapyscの文字列用のクラスです。このクラスは以下のコード例のようにコンストラクタの引数に`str`もしくは`String`型の値を受け付けます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String("Hello")\nassert string_1 == "Hello"\n\nstring_2: ap.String = ap.String(string_1)\nassert string_2 == "Hello"\n```': '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String("Hello")\nassert string_1 == "Hello"\n\nstring_2: ap.String = ap.String(string_1)\nassert string_2 == "Hello"\n```',  # noqa
    ##################################################
    "## String class interfaces": "## String クラスの各インターフェイス",
    ##################################################
    "For more details about the `String` class each interface, please see the following:": "`String`クラスの各インターフェイスの詳細については以下の資料などをご確認ください:",  # noqa
    ##################################################
    "- [String class comparison operations](string_comparison_operations.md)": "- [String クラスの比較制御](jp_string_comparison_operations.md)",  # noqa
    ##################################################
    "- [String class addition and multiplication operations](string_addition_and_multiplication.md)": "- [String クラスの加算と乗算の制御](jp_string_addition_and_multiplication.md)",  # noqa
    ##################################################
    "## String class constructor API": "## String クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "String class for apysc library.<hr>": "apyscライブラリにおける文字列用のクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: String or str": "- `value`: String or str",
    ##################################################
    "  - Initial string value.": "  - 文字列の値の初期値。",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "- `skip_init_substitution_expression_appending`: bool, default False": "- `skip_init_substitution_expression_appending`: bool, default False",  # noqa
    ##################################################
    "  - A boolean indicates whether to skip an initial substitution expression or not. This class uses this option internally.": "  - 初期値の代入のコード表現をスキップするかどうかの真偽値です。このオプションはクラス内部の実装で使用されます。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> string: ap.String = ap.String(\"Hello\")\n>>> string\nString('Hello')\n\n>>> string += \" World!\"\n>>> string\nString('Hello World!')\n```": "```py\n>>> import apysc as ap\n>>> string: ap.String = ap.String(\"Hello\")\n>>> string\nString('Hello')\n\n>>> string += \" World!\"\n>>> string\nString('Hello World!')\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [String class comparison operations](https://simon-ritchie.github.io/apysc/en/string_comparison_operations.html)": "- [String クラスの比較制御](https://simon-ritchie.github.io/apysc/jp/jp_string_comparison_operations.html)",  # noqa
    ##################################################
    "- [String class addition and multiplication operations](https://simon-ritchie.github.io/apysc/en/string_addition_and_multiplication.html)": "- [String クラスの加算と乗算の制御](https://simon-ritchie.github.io/apysc/jp/jp_string_addition_and_multiplication.html)",  # noqa
    ##################################################
    "## value property API": "## value 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current string value.<hr>": "現在の文字列の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `value`: str": "- `value`: str",
    ##################################################
    "  - Current string value.": "  - 現在の文字列の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> string: ap.String = ap.String("Hello")\n>>> string.value = "World!"\n>>> string.value\n\'World!\'\n```': '```py\n>>> import apysc as ap\n>>> string: ap.String = ap.String("Hello")\n>>> string.value = "World!"\n>>> string.value\n\'World!\'\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [apysc fundamental data classes value interface](https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html)": "- [apyscの基本的なデータクラスの value インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_fundamental_data_classes_value_interface.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_scale_x_and_y_from_point.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_scale_x_from_point and animation_scale_y_from_point interfaces": "# animation_scale_x_from_point と animation_scale_y_from_point のインターフェイス",  # noqa
    ##################################################
    "This page explains the `animation_scale_x_from_point` and `animation_scale_y_from_point` method interfaces.": "このページでは`animation_scale_x_from_point`と`animation_scale_y_from_point`の各メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `animation_scale_x_from_point` method interface will create an `ap.AnimationScaleXFromPoint` instance. You can animate the x-directional scale with it from a specified x-coordinate.": "`animation_scale_x_from_point`メソッドのインターフェイスは`ap.AnimationScaleXFromPoint`クラスのインスタンスを生成します。そのインスタンスを使って指定座標を基準としたX軸の拡縮のアニメーションを設定することができます。",  # noqa
    ##################################################
    "Similarly, the `animation_scale_y_from_point` method interface creates an `ap.AnimationScaleYFromPoint` instance. You can animate the y-directional scale with it from a specified y-coordinate.": "同じように`animation_scale_y_from_point`メソッドのインターフェイスでは`ap.AnimationScaleYFromPoint`クラスのインスタンスを生成します。そのインスタンスを使ってY軸の拡縮のアニメーションを設定することができます。",  # noqa
    ##################################################
    "These interfaces exist on a `GraphicsBase` subclass (that has the `scale_x_from_center` and `scale_y_from_center` interfaces), such as the `Rectangle` or `Circle`.": "これらのインターフェイスは`scale_x_from_center`や`scale_y_from_center`などのインターフェイスを持つ`Rectangle`や`Circle`などの`GraphicsBase`の各サブクラス上に存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the x-directional scale (left-side rectangle) and y-directional scale (right-side rectangle) animation (from 1.0 to 2.0) with the `animation_scale_x_from_point` and `animation_scale_y_from_point` methods.": "以下のコード例ではX軸の拡縮（1.0から2.0）を左側の四角へ、そしてY軸の拡縮を右側の四角へとそれぞれ`animation_scale_x_from_point`と`animation_scale_y_from_point`のメソッドを使って設定しています。",  # noqa
    ##################################################
    "These settings set the left-side rectangle scales from the left end (x=50) and the right-side rectangle scales from the bottom end (y=100).": "これらの設定は左側の四角では左端（x=50）の座標で、右側の四角では下端（y=100）の位置を基準に拡縮の設定を行っています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom enum import Enum\n\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\nLEFT_RECTANGLE_X: int = 50\nRIGHT_RECTANGLE_Y: int = 100\nSCALE_1: float = 1.0\nSCALE_2: float = 2.0\n\n\nclass Direction(Enum):\n    X = 1\n    Y = 2\n\n\nclass Options(TypedDict):\n    direction: Direction\n\n\ndef on_animation_complete_1(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_point(\n            scale_x_from_point=SCALE_1,\n            x=LEFT_RECTANGLE_X,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_point(\n            scale_y_from_point=SCALE_1,\n            y=RIGHT_RECTANGLE_Y,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n\n\ndef on_animation_complete_2(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_point(\n            scale_x_from_point=SCALE_2,\n            x=LEFT_RECTANGLE_X,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_point(\n            scale_y_from_point=SCALE_2,\n            y=RIGHT_RECTANGLE_Y,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50\n)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50\n)\n\noptions: Options = {"direction": Direction.X}\nleft_rectangle.animation_scale_x_from_point(\n    scale_x_from_point=SCALE_2,\n    x=LEFT_RECTANGLE_X,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\noptions = {"direction": Direction.Y}\nright_rectangle.animation_scale_y_from_point(\n    scale_y_from_point=SCALE_2,\n    y=RIGHT_RECTANGLE_Y,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\nap.save_overall_html(dest_dir_path="./animation_scale_x_and_y_from_point_basic_usage/")\n```': '```py\n# runnable\nfrom enum import Enum\n\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\nLEFT_RECTANGLE_X: int = 50\nRIGHT_RECTANGLE_Y: int = 100\nSCALE_1: float = 1.0\nSCALE_2: float = 2.0\n\n\nclass Direction(Enum):\n    X = 1\n    Y = 2\n\n\nclass Options(TypedDict):\n    direction: Direction\n\n\ndef on_animation_complete_1(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_point(\n            scale_x_from_point=SCALE_1,\n            x=LEFT_RECTANGLE_X,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_point(\n            scale_y_from_point=SCALE_1,\n            y=RIGHT_RECTANGLE_Y,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n\n\ndef on_animation_complete_2(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_point(\n            scale_x_from_point=SCALE_2,\n            x=LEFT_RECTANGLE_X,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_point(\n            scale_y_from_point=SCALE_2,\n            y=RIGHT_RECTANGLE_Y,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50\n)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50\n)\n\noptions: Options = {"direction": Direction.X}\nleft_rectangle.animation_scale_x_from_point(\n    scale_x_from_point=SCALE_2,\n    x=LEFT_RECTANGLE_X,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\noptions = {"direction": Direction.Y}\nright_rectangle.animation_scale_y_from_point(\n    scale_y_from_point=SCALE_2,\n    y=RIGHT_RECTANGLE_Y,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\nap.save_overall_html(dest_dir_path="./animation_scale_x_and_y_from_point_basic_usage/")\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_reverse.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_reverse interface": "# animation_reverse インターフェイス",
    ##################################################
    "This page explains the `animation_reverse` method interface.": "このページでは`animation_reverse`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `animation_reverse` interface reverses the running animations.": "`animation_reverse`インターフェイスは動いているアニメーションの再生を反転（逆再生）します。",  # noqa
    ##################################################
    "This interface exists in the instances that have the animation interfaces (such as the `animation_x`\\, `animation_move`).": "このインターフェイスは`animation_x`や`animation_move`などのアニメーション関係のインターフェイスを持つクラスのインスタンス上に存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the x-coordinate animation and starts the 1-second interval timer to reverse animation with the `animation_reverse` interface.": "以下の例ではX座標のアニメーションを設定し、1秒ごとの間隔で`animation_reverse`インターフェイスを使ってアニメーションを反転（逆作成）しています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer_1(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The event handler that timer calls after the 3 seconds.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.Timer(on_timer_2, delay=1000, options=options).start()\n\n\ndef on_timer_2(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The event handler that timer calls every second.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.animation_reverse()\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=400, duration=5000).start()\noptions: _RectOptions = {"rectangle": rectangle}\nap.Timer(on_timer_1, delay=3000, repeat_count=1, options=options).start()\n\nap.save_overall_html(dest_dir_path="animation_reverse_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer_1(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The event handler that timer calls after the 3 seconds.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.Timer(on_timer_2, delay=1000, options=options).start()\n\n\ndef on_timer_2(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The event handler that timer calls every second.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.animation_reverse()\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=400, duration=5000).start()\noptions: _RectOptions = {"rectangle": rectangle}\nap.Timer(on_timer_1, delay=3000, repeat_count=1, options=options).start()\n\nap.save_overall_html(dest_dir_path="animation_reverse_basic_usage/")\n```',  # noqa
    ##################################################
    "## Interface Notes": "## インターフェイスの特記事項",
    ##################################################
    "This interface can only use during animation. If you use this at the end of the animation, nothing happens, as follows:": "このインターフェイスはアニメーションが動いている間のみ使用することができます。以下のコード例のようにアニメーションが終了している状態で呼び出しても何も発生せずアニメーションが終了した状態のままとなります:",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n\n    # Nothing happens since the animation has already been completed.\n    rectangle.animation_reverse()\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=400, duration=1000).start()\n\noptions: _RectOptions = {"rectangle": rectangle}\nap.Timer(on_timer, delay=1500, repeat_count=1, options=options).start()\n\nap.save_overall_html(dest_dir_path="animation_reverse_notes/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n\n    # Nothing happens since the animation has already been completed.\n    rectangle.animation_reverse()\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=400, duration=1000).start()\n\noptions: _RectOptions = {"rectangle": rectangle}\nap.Timer(on_timer, delay=1500, repeat_count=1, options=options).start()\n\nap.save_overall_html(dest_dir_path="animation_reverse_notes/")\n```',  # noqa
    ##################################################
    "## animation_reverse API": "## animation_reverse API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Reverse all running animations.<hr>": "動いている全てのアニメーションを反転（逆再生）します。<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "Suppose you call this interface multiple times and animations reach the beginning or end of the animation. In that case, this interface ignores the reverse instruction. This behavior means that the same interval's timer tick reverse setting does not work correctly (since the same interval setting reaches the animation start).<hr>": "複数回このインターフェイスを呼び出した際などに、アニメーションの最初もしくは最後に到達しアニメーションが停止した場合、その後にこのインターフェイスを呼び出しても反転（逆再生）はされずに停止したままとなります。そのためアニメーション時間と同じ時間のタイマーなどで反転の指定した場合などは正常に動作しません。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_reverse()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_reverse()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: click.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# click interface": "# click インターフェイス",
    ##################################################
    "This page explains the `click` interface.": "このページでは`click`インターフェイスについて説明します。",
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `click` interface binds the click event to any `DisplayObject` instance (e.g., `Sprite`\\, `Rectangle`\\, and so on). These interfaces call the registered handler function if you mouse down on that instance and mouse up.": "`click`インターフェイスは`Sprite`や`Rectangle`クラスなどの任意の`DisplayObject`のサブクラスのインスタンスにクリックイベントを設定します。これらのインターフェイスで登録されたハンドラは対象のインスタンスをクリックした際に呼ばれます。",  # noqa
    ##################################################
    "Conversely, the `unbind_click` interface unbinds the click event from the `DisplayObject` instance.": "逆に`unbind_click`インターフェイスは対象の`DisplayObject`のインスタンスからクリックイベントの登録を解除します。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "The following page describes basic mouse event interfaces.": "以下のページでは基本的なマウスイベントのインターフェイスについて説明しています。",  # noqa
    ##################################################
    "- [Basic mouse event interfaces](mouse_event_basic.md)": "- [基本的なマウスイベントの各インターフェイス](jp_mouse_event_basic.md)",  # noqa
    ##################################################
    "## Basic usage of the click interface": "## click インターフェイスの基本的な使い方",
    ##################################################
    "Each `DisplayObject` instance has the `click` method, and you can bind handlers by that.": "`DisplayObject`のサブクラスの各インスタンスは`click`メソッドを持っており、そのインターフェイスを使ってイベントのハンドラを登録することができます。",  # noqa
    ##################################################
    "The following example binds the click event handler to the rectangle.": "以下のコード例では四角に対してクリックのイベントハンドラを設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(handler=on_click)\n\nap.save_overall_html(dest_dir_path="click_basic_usage_of_the_click_interface/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(handler=on_click)\n\nap.save_overall_html(dest_dir_path="click_basic_usage_of_the_click_interface/")\n```',  # noqa
    ##################################################
    "If you click the following rectangle, the rectangle color becomes the magenta color.": "もしも以下の四角をクリックすると四角の色はマゼンタに切り替わります。",  # noqa
    ##################################################
    "## Basic usage of the unbind_click interface": "## unbind_click インターフェイスの基本的な使い方",
    ##################################################
    "The `unbind_click` interface can remove the binded click event from a `DisplayObject` instance.": "`unbind_click`インターフェイスは`DisplayObject`のサブクラスのインスタンスから登録済みのクリックイベントの設定を取り除きます。",  # noqa
    ##################################################
    "The following example removes the click event by the `unbind_click` method, and nothing happens if you click the rectangle.": "以下のコード例では`unbind_click`メソッドでクリックイベントを取り除いているため四角をクリックしても何も発生しません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(handler=on_click)\nrectangle.unbind_click(handler=on_click)\n\nap.save_overall_html(dest_dir_path="click_basic_usage_of_the_unbind_click_interface/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(handler=on_click)\nrectangle.unbind_click(handler=on_click)\n\nap.save_overall_html(dest_dir_path="click_basic_usage_of_the_unbind_click_interface/")\n```',  # noqa
    ##################################################
    "## Unbind all the click event handlers": "## 全てのクリックのイベントハンドラを解除する",
    ##################################################
    "`unbind_click_all` interface can unbind all the click event handlers from the `DisplayObject` instance.": "`unbind_click_all`インターフェイスは`DisplayObject`のサブクラスのインスタンスから全ての登録されているクリックイベントのハンドラを解除します。",  # noqa
    ##################################################
    "The following example removes all the click events by the `unbind_click_all` method (if you click the rectangle, nothing happens).": "以下のコード例では`unbind_click_all`メソッドで全てのクリックイベントの設定を取り除いており、四角をクリックしても何も起きません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(handler=on_click)\nrectangle.unbind_click_all()\n\nap.save_overall_html(dest_dir_path="click_unbind_all_the_click_event_handlers/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(handler=on_click)\nrectangle.unbind_click_all()\n\nap.save_overall_html(dest_dir_path="click_unbind_all_the_click_event_handlers/")\n```',  # noqa
    ##################################################
    "## click API": "## click API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add a click event listener setting.<hr>": "クリックイベントのリスナー設定を追加します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - A callable would be called when clicking this instance.": "  - このインスタンスをクリックした際に呼ばれるハンドラ。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `name`: str": "- `name`: str",
    ##################################################
    "  - Handler's name.": "  - ハンドラ名。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.x += 10\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.x += 10\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## unbind_click API": "## unbind_click API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind specified handler's click event.<hr>": "指定されたクリックイベントのハンドラー設定を取り除きます。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Unbinding target Callable.": "  - イベント設定を取り除く対象の関数やメソッドなど。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n...     rectangle.unbind_click(on_click)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n...     rectangle.unbind_click(on_click)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```',  # noqa
    ##################################################
    "## unbind_click_all API": "## unbind_click_all API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind all click events.<hr>": "全てのクリックイベント設定を取り除きます。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n...     rectangle.unbind_click_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n...     rectangle.unbind_click_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```',  # noqa
}



"""This module is for the SVG's `3D bezier curve` (C) path
data class implementation.
"""

from typing import Any
from typing import Union

from typing_extensions import final

from apysc._geom.path_control_x1_mixin import PathControlX1MixIn
from apysc._geom.path_control_x2_mixin import PathControlX2MixIn
from apysc._geom.path_control_y1_mixin import PathControlY1MixIn
from apysc._geom.path_control_y2_mixin import PathControlY2MixIn
from apysc._geom.path_data_base import PathDataBase
from apysc._geom.path_dest_x_mixin import PathDestXMixIn
from apysc._geom.path_dest_y_mixin import PathDestYMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class PathBezier3D(
    PathDataBase,
    PathDestXMixIn,
    PathDestYMixIn,
    PathControlX1MixIn,
    PathControlY1MixIn,
    PathControlX2MixIn,
    PathControlY2MixIn,
):
    """
    Path data class for the SVG's `3D bezier curve` (C).

    References
    ----------
    - Path class
        - https://simon-ritchie.github.io/apysc/en/path.html
    - Graphics draw_path interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html
    - PathBezier3D class
        - https://simon-ritchie.github.io/apysc/en/path_bezier_3d.html
    - PathBezier3DContinual class
        - https://simon-ritchie.github.io/apysc/en/path_bezier_3d_continual.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color="#fff", thickness=3)
    >>> path: ap.Path = sprite.graphics.draw_path(
    ...     path_data_list=[
    ...         ap.PathMoveTo(x=0, y=50),
    ...         ap.PathBezier3D(
    ...             control_x1=0,
    ...             control_y1=0,
    ...             control_x2=50,
    ...             control_y2=0,
    ...             dest_x=50,
    ...             dest_y=50,
    ...         ),
    ...         ap.PathBezier3DContinual(
    ...             control_x=100, control_y=100, dest_x=100, dest_y=50
    ...         ),
    ...     ]
    ... )
    """

    @final
    @arg_validation_decos.is_integer(arg_position_index=1)
    @arg_validation_decos.is_integer(arg_position_index=2)
    @arg_validation_decos.is_integer(arg_position_index=3)
    @arg_validation_decos.is_integer(arg_position_index=4)
    @arg_validation_decos.is_integer(arg_position_index=5)
    @arg_validation_decos.is_integer(arg_position_index=6)
    @arg_validation_decos.is_boolean(arg_position_index=7)
    @arg_validation_decos.is_builtin_string(arg_position_index=8, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        control_x1: Union[int, Int],
        control_y1: Union[int, Int],
        control_x2: Union[int, Int],
        control_y2: Union[int, Int],
        dest_x: Union[int, Int],
        dest_y: Union[int, Int],
        *,
        relative: Union[bool, Boolean] = False,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Path data class for the SVG's `3D bezier curve` (C).

        Parameters
        ----------
        control_x1 : Int or int
            X-coordinate of the bezier's first control point.
        control_y1 : Int or int
            Y-coordinate of the bezier's first control point.
        control_x2 : Int or int
            X-coordinate of the bezier's second control point.
        control_y2 : Int or int
            Y-coordinate of the bezier's second control point.
        dest_x : Int or int
            X-coordinate of the destination point.
        dest_y : Int or int
            Y-coordinate of the destination point.
        relative : bool or Boolean, default False
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.

        References
        ----------
        - Path class
            - https://simon-ritchie.github.io/apysc/en/path.html
        - Graphics draw_path interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html
        - PathBezier3D class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_3d.html
        - PathBezier3DContinual class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_3d_continual.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color="#fff", thickness=3)
        >>> path: ap.Path = sprite.graphics.draw_path(
        ...     path_data_list=[
        ...         ap.PathMoveTo(x=0, y=50),
        ...         ap.PathBezier3D(
        ...             control_x1=0,
        ...             control_y1=0,
        ...             control_x2=50,
        ...             control_y2=0,
        ...             dest_x=50,
        ...             dest_y=50,
        ...         ),
        ...         ap.PathBezier3DContinual(
        ...             control_x=100, control_y=100, dest_x=100, dest_y=50
        ...         ),
        ...     ]
        ... )
        """
        from apysc._geom.path_label import PathLabel

        self._variable_name_suffix = variable_name_suffix
        super(PathBezier3D, self).__init__(
            path_label=PathLabel.BEZIER_3D, relative=relative
        )
        self.control_x1 = self._get_copied_int_from_builtin_val(
            integer=control_x1, attr_identifier="control_x1"
        )
        self.control_y1 = self._get_copied_int_from_builtin_val(
            integer=control_y1, attr_identifier="control_y1"
        )
        self.control_x2 = self._get_copied_int_from_builtin_val(
            integer=control_x2, attr_identifier="control_x2"
        )
        self.control_y2 = self._get_copied_int_from_builtin_val(
            integer=control_y2, attr_identifier="control_y2"
        )
        self.dest_x = self._get_copied_int_from_builtin_val(
            integer=dest_x, attr_identifier="dest_x"
        )
        self.dest_y = self._get_copied_int_from_builtin_val(
            integer=dest_y, attr_identifier="dest_y"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _get_svg_str(self) -> str:
        """
        Get a path's SVG string created with the current setting.

        Returns
        -------
        svg_str : str
            An SVG path string was created with the current setting.
        """
        from apysc._type.value_util import get_value_str_for_expression

        svg_char: String = self._get_svg_char()
        svg_char_str: str = get_value_str_for_expression(value=svg_char)
        control_x1_str: str = get_value_str_for_expression(value=self._control_x1)
        control_y1_str: str = get_value_str_for_expression(value=self._control_y1)
        control_x2_str: str = get_value_str_for_expression(value=self._control_x2)
        control_y2_str: str = get_value_str_for_expression(value=self._control_y2)
        dest_x_str: str = get_value_str_for_expression(value=self._dest_x)
        dest_y_str: str = get_value_str_for_expression(value=self._dest_y)
        svg_str: str = (
            f"{svg_char_str} + String({control_x1_str}) "
            f'+ " " + String({control_y1_str}) '
            f'+ " " + String({control_x2_str}) '
            f'+ " " + String({control_y2_str}) '
            f'+ " " + String({dest_x_str}) '
            f'+ " " + String({dest_y_str})'
        )
        return svg_str

    @final
    @arg_validation_decos.is_integer(arg_position_index=1)
    @arg_validation_decos.is_integer(arg_position_index=2)
    @arg_validation_decos.is_integer(arg_position_index=3)
    @arg_validation_decos.is_integer(arg_position_index=4)
    @arg_validation_decos.is_integer(arg_position_index=5)
    @arg_validation_decos.is_integer(arg_position_index=6)
    @arg_validation_decos.is_boolean(arg_position_index=7)
    @add_debug_info_setting(module_name=__name__)
    def update_path_data(
        self,
        control_x1: Union[int, Int],
        control_y1: Union[int, Int],
        control_x2: Union[int, Int],
        control_y2: Union[int, Int],
        dest_x: Union[int, Int],
        dest_y: Union[int, Int],
        *,
        relative: Union[bool, Boolean] = False,
    ) -> None:
        """
        Update the path's data settings.

        Parameters
        ----------
        control_x1 : Int or int
            X-coordinate of the bezier's first control point.
        control_y1 : Int or int
            Y-coordinate of the bezier's first control point.
        control_x2 : Int or int
            X-coordinate of the bezier's second control point.
        control_y2 : Int or int
            Y-coordinate of the bezier's second control point.
        dest_x : Int or int
            X-coordinate of the destination point.
        dest_y : Int or int
            Y-coordinate of the destination point.
        relative : bool or Boolean, default False
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).

        Examples
        --------
        >>> import apysc as ap
        >>> bezier_3d_continual = ap.PathBezier3D(
        ...     control_x1=0,
        ...     control_y1=0,
        ...     control_x2=50,
        ...     control_y2=0,
        ...     dest_x=50,
        ...     dest_y=50,
        ... )
        >>> bezier_3d_continual.update_path_data(
        ...     control_x1=100,
        ...     control_y1=100,
        ...     control_x2=150,
        ...     control_y2=100,
        ...     dest_x=150,
        ...     dest_y=150,
        ... )
        >>> bezier_3d_continual.control_x1
        Int(100)

        >>> bezier_3d_continual.control_y1
        Int(100)

        >>> bezier_3d_continual.control_x2
        Int(150)

        >>> bezier_3d_continual.control_y2
        Int(100)

        >>> bezier_3d_continual.dest_x
        Int(150)

        >>> bezier_3d_continual.dest_y
        Int(150)
        """
        self.control_x1 = self._get_copied_int_from_builtin_val(
            integer=control_x1, attr_identifier="control_x1"
        )
        self.control_y1 = self._get_copied_int_from_builtin_val(
            integer=control_y1, attr_identifier="control_y1"
        )
        self.control_x2 = self._get_copied_int_from_builtin_val(
            integer=control_x2, attr_identifier="control_x2"
        )
        self.control_y2 = self._get_copied_int_from_builtin_val(
            integer=control_y2, attr_identifier="control_y2"
        )
        self.dest_x = self._get_copied_int_from_builtin_val(
            integer=dest_x, attr_identifier="dest_x"
        )
        self.dest_y = self._get_copied_int_from_builtin_val(
            integer=dest_y, attr_identifier="dest_y"
        )
        self.relative = self._get_copied_boolean_from_builtin_val(
            bool_val=relative, attr_identifier="relative"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap

        if not isinstance(other, PathBezier3D):
            result: ap.Boolean = ap.Boolean(
                False, variable_name_suffix=self._variable_name_suffix
            )
            return result
        return (
            self.control_x1 == other.control_x1
            and self.control_y1 == other.control_y1
            and self.control_x2 == other.control_x2
            and self.control_y2 == other.control_y2
            and self.dest_x == other.dest_x
            and self.dest_y == other.dest_y
            and self.relative == other.relative
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ne__(self, other: Any) -> Any:
        """
        Not equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap

        result: ap.Boolean = self == other
        result = result.not_
        return result
